"""audio - Manages sounds and music in the game."""

from random import choice
import os.path

from direct.showbase.DirectObject import DirectObject
from direct.showbase.Audio3DManager import Audio3DManager

class Audio(object):

    event_handler = DirectObject()
    sound_path = 'sounds/'
    sound_names = ('stone_grinding.mp3', 'pendulum_whoosh.mp3', 'gate_opening.mp3')
    sounds = {}
    sound_events = {}
    current_sounds = []
    sound_volume = 1.0
    music_path = 'music/'
    music_names = ('snake_valley.mp3', 'walls_closing_in.mp3')
    music = {}
    blacklist = []
    current_track = None
    music_volume = 0.75
    manager3d = None

    @classmethod
    def init_3d(cls, cam):
        #cls.manager3d = Audio3DManager(base.sfxManagerList[0], cam)
        pass

    @classmethod
    def set_music_volume(cls, volume):
        assert volume >= 0.0 and volume <= 1.0
        cls.music_volume = volume

    @classmethod
    def set_sound_volume(cls, volume):
        assert volume >= 0.0 and volume <= 1.0
        cls.sound_volume = volume

    @classmethod
    def load_sounds(cls):
        for name in cls.sound_names:
            sound = base.loader.loadSfx(cls.sound_path + name)
            name = os.path.splitext(name)[0]
            cls.sounds[name] = sound

    @classmethod
    def load_music(cls):
        for name in cls.music_names:
            musicFile = cls.music_path + name.replace(' ', '_')
            track = base.loader.loadMusic(musicFile)
            name = os.path.splitext(name)[0]
            cls.music[name] = track

    @classmethod
    def blacklist_track(cls, exclusion):
        assert exclusion in cls.music
        if not exclusion in cls.blacklist:
            cls.blacklist.append(exclusion)

    @classmethod
    def play_track(cls, track, loop=False, volume=None, position=None):
        if volume is None:
            volume = Audio.music_volume
        assert track in cls.music
        if cls.current_track is not None:
            cls.music[cls.current_track].stop()
        if isinstance(loop, int):
            if not loop:
                cls.music[track].set_loop(False)
            else:
                cls.music[track].set_loop_count(loop)
        elif isinstance(loop, bool):
            cls.music[track].set_loop(loop)
        if volume is not cls.music_volume:
            assert isinstance(volume, int) or isinstance(volume, float)
        cls.music[track].setVolume(volume)
        if position is not None:
            cls.music[track].setTime(position)
        cls.music[track].play()
        cls.current_track = track

    @classmethod
    def stop_track(cls):
        if not (cls.current_track and cls.music[cls.current_track].status() == 2):
            print("WARNING! The following did NOT pass:")
            print("cls.current_track and cls.music[cls.current_track].status() == 2")
            print("This may very well mean that the audio is broken!")
        cls.music[cls.current_track].stop()
        cls.current_track = None

    @classmethod
    def play_random_track(cls, loop=False, volume=None):
        if volume is None:
            volume = cls.music_volume
        random_track = cls.select_random_track()
        cls.play_track(random_track, loop, volume)

    @classmethod
    def select_random_track(cls):
        random_track = choice(list(cls.music.keys()))
        if random_track in cls.blacklist:
            random_track = cls.select_random_track()
        return random_track

    @classmethod
    def add_sound_event(cls, event, sound, volume=None, loop=False, playRate=1.0):
        if volume is None:
            volume = cls.sound_volume
        assert isinstance(event, str)
        assert sound in cls.sounds or sound is None
        if not event in cls.sound_events:
            cls.sound_events[event] = []
        assert isinstance(cls.sound_events[event], list)
        cls.sound_events[event].append(sound)
        cls.event_handler.accept(event, cls.play_sound, [sound, volume, loop,
                                                       playRate])

    @classmethod
    def play_sound(cls, sound, volume=None, loop=False, playRate=1.0, object=None):
        if volume is None:
            volume = cls.sound_volume
        assert sound in cls.sounds
        if isinstance(loop, int):
            if not loop:
                cls.sounds[sound].set_loop(False)
            else:
                cls.sounds[sound].set_loop(loop)
        elif isinstance(loop, bool):
            cls.sounds[sound].set_loop(loop)
        if volume is not cls.sound_volume:
            assert isinstance(volume, int) or isinstance(volume, float)
        cls.sounds[sound].setVolume(volume)
        assert isinstance(playRate, int) or isinstance(playRate, float)
        cls.sounds[sound].setPlayRate(playRate)
        cls.sounds[sound].setFinishedEvent('finished' + sound)
        cls.event_handler.accept('finished' + sound, cls.stop_sound, [sound])
        if cls.sounds[sound].status() != 2:
            cls.sounds[sound].play()
            cls.current_sounds.append(sound)

        #if object is not None:
        #    cls.manager3d.attachSoundToObject(cls.sounds[sound], object)

    @classmethod
    def stop_sound(cls, sound):
        assert sound in cls.current_sounds
        cls.sounds[sound].stop()
        cls.current_sounds.remove(sound)

    @classmethod
    def stop_all_sounds(cls):
        for sound in cls.current_sounds:
            cls.sounds[sound].stop()
            cls.current_sound.remove(sound)