from direct.showbase.ShowBase import ShowBase
from panda3d.core import ModifierButtons, get_model_path, loadPrcFile, WindowProperties
from panda3d.core import TextPropertiesManager, TextProperties, AntialiasAttrib

import sys

if sys.platform == 'win32':
    try:
        import ctypes
    except:
        ctypes = None
        print("ctypes import failed")

class Engine(ShowBase):
    """
    This class derives from Panda3D's ShowBase class. It is intended to act as
    the game's liaison with the Panda3D API.

    """

    def __init__(self):
        loadPrcFile('config.prc')

        ShowBase.__init__(self)
        # Disable modifier compound events to handle mod keys independently
        self.mouseWatcherNode.setModifierButtons(ModifierButtons())
        self.buttonThrowers[0].node().setModifierButtons(ModifierButtons())
        self.setBackgroundColor((0, 0, 0, 1))
        self.disableMouse()
        self.camLens.set_fov(60)
        self.camLens.set_near(0.1)
        self.camLens.set_far(100)

        self.mouse_sensitivity = 1.0

        self.cursor_on = True

        self.render.set_antialias(AntialiasAttrib.M_auto)

        self.path = get_model_path()
        self.path.prepend_directory("models")
        self.path.prepend_directory("textures")

        #self.accept(';', self.oobe)
        #self.accept('g', self.generate_cube_maps)
        #self.accept('s', self.screenshot)

        # Text properties.
        tp_key = TextProperties()
        tp_key.set_text_color(1.0, 1.0, 0.5, 1.0)
        tp_mgr = TextPropertiesManager.get_global_ptr()
        tp_mgr.set_properties('key', tp_key)

    def map_vsc(self, vsc, default):
        """ Map scan code to an event to listen to.  The default argument is what is
        returned if we don't have a way to map the scan code to an event name. """

        if sys.platform == 'win32' and ctypes is not None:
            vk = ctypes.windll.user32.MapVirtualKeyA(vsc, 1) # MAPVK_VSC_TO_VK
            if vk > 0:
                # OK, we've got the vk, now map it to a character.
                char = ctypes.windll.user32.MapVirtualKeyA(vk, 2) # MAPVK_VK_TO_CHAR
                if char > 0:
                    return chr(char).lower()

        return default

    def centre_cursor(self):
        if self.cursor_on:
            return
        return self.win.move_pointer(0, base.win.get_x_size() / 2,
                                        base.win.get_y_size() / 2)

    def get_mouse_delta(self):
        if self.cursor_on:
            return 0.0, 0.0
        cursor_pos = base.win.get_pointer(0)
        delta_x = cursor_pos.get_x() - (self.win.get_x_size() / 2)
        delta_y = cursor_pos.get_y() - (self.win.get_y_size() / 2)
        self.centre_cursor()
        return delta_x * self.mouse_sensitivity, delta_y * self.mouse_sensitivity

    def toggle_cursor(self, override=None):
        curProps = self.win.getProperties()
        currentSetting = curProps.getCursorHidden()
        if override is not None:
            newSetting = override
        else:
            newSetting = not currentSetting
        self.cursor_on = newSetting

        newProps = WindowProperties()
        newProps.setCursorHidden(not newSetting)
        if newSetting:
            newProps.setMouseMode(WindowProperties.MAbsolute)
        else:
            newProps.setMouseMode(WindowProperties.MRelative)
            newProps.setForeground(True)
        self.win.requestProperties(newProps)
        if not newSetting:
            self.centre_cursor()

    def toggle_fullscreen(self, override=None):
        curProps = self.win.getProperties()
        currentSetting = curProps.getFullscreen()
        if override is not None:
            newSetting = override
        else:
            newSetting = not currentSetting
        if newSetting is not currentSetting:
            newProps = WindowProperties()
            newProps.setFullscreen(newSetting)
            self.win.requestProperties(newProps)

    def generate_cube_maps(self):
        render.set_color_scale((0.2, 0.2, 0.2, 1.0))
        self.saveCubeMap('textures/orb_env_#.png', size=64)
