"""mainmenu - Menu that appears at the start of the game."""

from sys import exit

from direct.gui.DirectGui import DGG, DirectFrame, DirectLabel, DirectButton
from direct.gui.DirectGui import DirectSlider
from direct.fsm.FSM import FSM
from direct.gui.OnscreenText import OnscreenText
from direct.gui.OnscreenImage import OnscreenImage

from audio import Audio
from engine import Engine


class MainMenu(FSM):

    def __init__(self, parent):
        FSM.__init__(self, 'MainMenuFSM')
        self.parent = parent
        # Main frame: Start Game, Settings, Exit
        self.main_frame = DirectFrame(frameColor=(0, 0, 0, 0.5),
                                     frameSize=(-1.2, 1.2, -0.65, 0.7),
                                     pos=(0, -1, 0), relief=DGG.FLAT,
                                     borderWidth=(0.11, 0.1))
        self.title = DirectLabel(text='Arcanum Essentia Obcurata:\nNemesis', 
                                 relief=None,
                                 frameSize=(-0.5, 0.5, -0.1, 0.1),
                                 text_scale=(0.17, 0.17), pos=(0, 0, 0.45),
                                 text_fg=(0.79, 0.69, 0.57, 1))
        self.title.reparent_to(self.main_frame)
        self.start_game = DirectButton(text='Start Game', relief=DGG.FLAT,
                                      frameSize=(-0.4, 0.4, -0.06, 0.12),
                                      frameColor=(0,0,0,0.5),
                                      pos=(0, 0, -0.25), text_scale=(0.15, 0.15),
                                      text_pos=(0, -0.01),
                                      text_fg=(0.79, 0.69, 0.57, 1),
                                      command=self.game_start)
        self.start_game.reparent_to(self.main_frame)
        '''self.settings = DirectButton(text='Settings', relief=None,
                                     frameSize=(-0.2, 0.2, -0.05, 0.05),
                                     pos=(0, 0, -0.05), text_scale=(0.15, 0.15),
                                     text_fg=(0.79, 0.69, 0.57, 1),
                                     command=self.request,
                                     extraArgs=['Settings'])
        self.settings.reparent_to(self.main_frame)'''
        self.exit_game = DirectButton(text='Exit', relief=DGG.FLAT,
                                     frameSize=(-0.4, 0.4, -0.05, 0.13),
                                     frameColor=(0,0,0,0.5),
                                     pos=(0, 0, -0.5), text_scale=(0.15, 0.15),
                                     text_fg=(0.79, 0.69, 0.57, 1),
                                     command=exit)
        self.exit_game.reparent_to(self.main_frame)

        self.mouse_sensitivity = DirectSlider(range=(0.25, 4.0), value=1.0, pageSize=3,
                                        frameColor=(0.54, 0.37, 0.18, 1),
                                        text='mouse sensitivity:',
                                        text_scale=0.08,
                                        text_pos=(-0.65,0),
                                        text_fg=(0.79, 0.69, 0.57, 1),
                                        frameSize=(-0.2, 0.6, -0.05, 0.05),
                                        pos=(0, 0, 0.1),
                                        command=self.set_mouse_sensitivity,
                                        extraArgs=[])
        self.mouse_sensitivity.reparent_to(self.main_frame)

        # Settings frame: Fullscreen, Music Volume, Sound Volume
        self.settings_frame = DirectFrame(frameColor=(0.26, 0.18, 0.06, 1.0),
                                         frameSize=(-1, 1, -0.75, 0.75),
                                         pos=(0, -1, 0), relief=DGG.RIDGE,
                                         borderWidth=(0.11, 0.1))
        self.fullscreen_label = DirectLabel()
        self.fullscreen_label.reparent_to(self.settings_frame)
        self.fullscreen = DirectButton(text='Off', relief=None,
                                       frameSize=(-0.2, 0.2, -0.05, 0.05),
                                       pos=(0.7, 0, 0), text_scale=(0.15, 0.15),
                                       text_fg=(0.79, 0.69, 0.57, 1),
                                       command=self.set_fullscreen)
        self.fullscreen.reparent_to(self.settings_frame)
        self.music_label = DirectLabel()
        self.music_label.reparent_to(self.settings_frame)
        self.music_volume = DirectSlider(range=(0,100), value=50, pageSize=3,
                                        frameColor=(0.54, 0.37, 0.18, 1),
                                        frameSize=(-0.2, 0.2, -0.05, 0.05),
                                        pos=(0, 0, 0.1),
                                        command=self.set_volume,
                                        extraArgs=['music'])
        self.music_volume.reparent_to(self.settings_frame)
        self.to_main = DirectButton(text='Back', relief=None,
                                   frameSize=(-0.2, 0.2, -0.05, 0.05),
                                   pos=(0, 0, -0.5), text_scale=(0.15, 0.15),
                                   text_fg=(0.79, 0.69, 0.57, 1),
                                   command=self.request, extraArgs=['Main'])
        self.to_main.reparent_to(self.settings_frame)

        self.request('Main')

    def set_mouse_sensitivity(self):
        self.parent.engine.mouse_sensitivity = self.mouse_sensitivity['value']

    def set_volume(self, type):
        if type == 'music':
            Audio.set_music_volume(self.music_volume['value'] / 100.0)
        elif type == 'sound':
            Audio.set_sound_volume(self.sound_volume['value'] / 100.0)

    def set_fullscreen(self):
        Engine.toggle_fullscreen()
        if self.fullscreen['text'] == 'Off':
            self.fullscreen['text'] = 'On'
        else:
            self.fullscreen['text'] = 'Off'

    def game_start(self):
        insruction_text = OnscreenText('Loading...', mayChange=True,
                                       pos=(0, -0.9), scale=0.1,
                                       fg=(1, 1, 1, 1), bg=(0, 0, 0, 0.5))
        insruction_text.set_transparency(1)
        base.graphicsEngine.render_frame()
        base.graphicsEngine.render_frame()
        self.parent.hide_menu()
        insruction_text.destroy()

    def destroy(self):
        self.main_frame.destroy()
        self.settings_frame.destroy()

    def enterMain(self):
        self.settings_frame.hide()
        self.main_frame.show()

    def enterSettings(self):
        self.main_frame.hide()
        self.settings_frame.show()