from panda3d.core import Plane, PlaneNode, TransparencyAttrib, Texture, Vec3, Point3, NodePath, TextureStage, CullFaceAttrib, BitMask32

class Water():
    def __init__(self, root):
        self.root = root
        
        self.root.set_shader_off(1)
        self.root.set_light_off(1)
        self.root.set_color_scale(0.5, 0.6, 0.8, 0.7)
        self.root.set_transparency(TransparencyAttrib.M_alpha, 10)

        wbuffer = base.win.make_texture_buffer("water", 512, 512)
        wbuffer.set_clear_color_active(True)
        wbuffer.set_clear_color((1.0, 1.0, 1.0, 1.0))

        self.camera = base.makeCamera(wbuffer)
        self.camera.node().set_lens(base.camLens)
        self.camera.node().set_camera_mask(BitMask32.bit(1))
        
        self.root.hide(BitMask32.bit(1))

        texture = wbuffer.get_texture()
        texture.set_wrap_u(Texture.WM_clamp)
        texture.set_wrap_v(Texture.WM_clamp)
        texture.set_minfilter(Texture.FT_linear_mipmap_linear)
        
        self.plane = Plane(Vec3(0, 0, 1), Point3(0, 0, self.root.get_z()))
        self.plane_node = render.attach_new_node(PlaneNode("water", self.plane))

        self.camera.reparent_to(self.plane_node)

        tmpnp = NodePath("state-init")
        tmpnp.set_clip_plane(self.plane_node)
        tmpnp.set_attrib(CullFaceAttrib.make_reverse())

        self.camera.node().set_initial_state(tmpnp.get_state())
        self.root.projectTexture(TextureStage("reflection"), texture, self.camera)

        taskMgr.add(self.update_camera, 'update-water', sort=45)

    def update_camera(self, task):
        mat = base.cam.get_mat(self.plane_node)
        mat *= self.plane.get_reflection_mat()
        self.camera.set_mat(mat)
        return task.cont
