from game.audio import Audio
from game.engine import Engine
from game.game import Game
from game.mainmenu import MainMenu

from panda3d.core import CardMaker

class Main():
    def __init__(self):
        self.engine = Engine()
        Audio.load_sounds()
        Audio.load_music()
        self.main_menu = MainMenu(parent=self)
        Audio.play_track('snake_valley', loop=True)
        self.game = None
    
    def start_game(self):
        self.game = Game(self.engine)

        # Upload the scene to the GPU to prevent later chugs
        self.game.world.root.prepare_scene(self.engine.win.get_gsg())

        # HACK to be black
        cm = CardMaker('')
        cm.set_frame(-1, 1, -1, 1)
        self.card = render2d.attach_new_node(cm.generate())
        self.card.set_color((0, 0, 0, 1))
        base.cam.set_h(-180)
        self.engine.taskMgr.add(self.start_task, 'start_game')

        self.engine.accept('show-menu', self.show_menu)

    def start_task(self, task):
        if base.cam.get_h() >= 0:
            self.card.hide()
            self.hide_menu()
            return task.done

        # HACK to prevent chugs
        base.cam.set_h(base.cam.get_h() + 10)
        return task.cont

    def show_menu(self):
        self.engine.toggle_cursor(True)
        self.main_menu.main_frame.show()
        self.engine.accept('escape', self.hide_menu)

    def hide_menu(self):
        if self.game is None:
            self.start_game()

        self.main_menu.main_frame.hide()
        self.engine.toggle_cursor(False)
        self.engine.accept('escape', self.show_menu)

    def run(self):
        self.engine.run()

#if __name__ == "__main__":
app = Main()
app.run()
